# -*- coding: utf-8 -*-

def version():
    print("Version: 1.0.84")
    print("Security Investment Analysis Tool (SIAT)")
    print("Find a bug? Write to wdehong2000@163.com")
    print("Copyrighs (C) WANG Dehong, 2021")
    print("Not for commercial use unless authorised")

    
def author():
    print("Author: Dr. WANG Dehong, 王德宏")
    print("Professor, International Business School")
    print("Beijing Foreign Studies University")
    print("Email: wdehong2000@163.com")
    print("Room 618 International Business School Building, West Campus")
    print("No.19 West 3rd Ring North Road, Haidian District")
    print("Beijing, P. R. China. Zip code: 100089")