# -*- coding: utf-8 -*-

# 绝对引用指定目录中的模块
import sys
sys.path.insert(0,r'S:\siat\siat')
from option_china import *
#==============================================================================
#查找中国商品期权的常见品种
df1=option_comm_china()

#查找中国黄金期权的可用合约
df2=option_comm_china('黄金')

#查找中国黄金期权au2112和au2202的具体合约（看涨/看跌合约）
df3=option_comm_china('黄金','au2112')
df3b=option_comm_china('黄金','au2202')
#同一时刻行权价对期权合约的影响：行权价越高，看涨期权合约价格越低，看跌期权合约价越高

#单一期权合约价格的运动趋势：默认不带趋势线
df4=option_comm_trend_china('au2202C364','2021-9-1','2021-9-30',power=4)

#期权方向对合约价格走势的影响：看涨/看跌期权合约的价格走势正好相反
df5=option_comm_trend_china(['au2202C364','au2202P364'],'2021-8-1','2021-9-30')

#行权价对合约价格时间序列走势的影响：看涨期权，行权价低者合约价高
df6=option_comm_trend_china(['au2112C328','au2112C364'],'2021-8-1','2021-9-30')
#行权价对合约价格时间序列走势的影响：看跌期权，行权价低者合约价低，与看涨期权正好相反
df6b=option_comm_trend_china(['au2112P328','au2112P364'],'2021-8-1','2021-9-30')

#到期时间对合约价格走势的影响：看涨期权，到期日近者合约价低（时间价值少）
df7=option_comm_trend_china(['au2112C364','au2202C364'],'2021-8-1','2021-9-30')
#到期时间对合约价格走势的影响：看跌期权，到期日近者合约价低，与看涨期权一致
df7b=option_comm_trend_china(['au2112P364','au2202P364'],'2021-8-1','2021-9-30')



#==============================================================================
